package ps3;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

public class Set<T> implements Iterable<T>, Collection<T> {
	/**  OVERVIEW: A Set is a mutable, unbounded set of objects.  
	 *      A typical Set is {x_1, ..., x_n }.
	 */
	
	// Rep:
	private Vector<T> elements;
	
	/**
	 * EFFECTS: Initializes this to an empty set: { }.
	 */
	public Set() {
		elements = new Vector<T>();
	}
	
	/**
	 * EFFECTS: Initializes this to a set containing the same elements
	 *    as the set <tt>s</tt> (a shallow copy).  
	 * @param s
	 */
	public Set(Set<T> s) {
		elements = new Vector<T>(s.elements);
	}
	
	/**
	 * MODIFIES: this<br>
	 * EFFECTS: Adds <code>el</code> to the elements of <code>this</code>:
	 *    <code>this_post = this_pre U { el }</code>
	 *    Returns true iff <code>el</code> was not an element of <code>this_pre</code>.
	 * @param el
	 */
	public boolean add(T el) {
		if (!elements.contains(el)) {
			elements.add(el);
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * MODIFIES: this<br>
	 * EFFECTS: Adds all elements of <code>c</code> to <code>this</code>.
	 *    Returns true iff any element of <code>c</code> is not in <code>this_pre</code>.
	 */
	public boolean addAll(Collection<? extends T> c) {
		boolean res = false;
		for (T el : c) {
			if (add(el)) {
				res = true;
			}
		}
		return res;
	}

	/**
	 * EFFECTS: Returns an iterator that yields each element of <code>this</code>
	 *    once.<br>
	 * postREQUIRES: this must not be modified while the iterator is in use. 
	 */
	public Iterator<T> iterator() {
		return elements.iterator();
	}
	
	/**
	 * REQUIRES: <code>this</code> has at least one element<br>
	 * EFFECTS: Returns an element of <code>this</code>.
	 */
	public T choose() {
		return elements.firstElement();
	}
	
    /**
     * EFFECTS: Returns true iff <code>el</code> is an element of <code>this</code>. 
     * @param el
     */
	public boolean contains(Object el) {
		return elements.contains(el);
	}

	/**
	 * EFFECTS: Returns the number of elements in <code>this</code>.
	 */
	public int size() {
		return elements.size();
	}

	/**
	 *  EFFECTS: Returns true iff <code>this</code> has no elements.
	 */
	public boolean isEmpty() {
		return elements.isEmpty();
	}
	
	/**
	 * MODIFIES: <code>this</code><br>
	 * EFFECTS: Removes <code>el</code> from <code>this</code>:
	 *    <code>this_post = this_pre - { el }</code>
	 *    Returns <code>el \isin this_pre</code>
	 * @param el
	 * @return
	 */
	public boolean remove(Object el) {
		return elements.remove(el);
	}

	// Shouldn't need both of these...
	public void subtract(Set<T> u) 
	// MODIFIES: this
	// EFFECTS: Removes all elements of u from this.
	{
		for (T el : u.elements) {
			remove (el);
		}
	}

	public void subtract(Vector<T> u) 
	// MODIFIES: this
	// EFFECTS: Removes all elements of u from this.
	{
		for (T el : u) {
			remove (el);
		}
	}

	public void union(Set<T> t)
	// MODIFIES: this
	// EFFECTS: this_post = this_pre U t 
	{
		for (T el : t.elements) {
			add(el);
		}
	}
	
	public String toString() {
		String res = "{";
		boolean firstone = true;
		for (T el : elements) {
			if (firstone) {
				firstone = false;
			} else {
				res += ", ";
			}
			res += el.toString();
		}
		res += "}";
		return res;
	}

	/**
	 * MODIFIES: <code>this</code><br>
	 * EFFECTS: <code>this_post = { }</code>
	 */
	public void clear() {
		elements.clear();
	}

	/**
	 * EFFECTS: Returns <code>true</code> if every element of <code>c</code>
	 *    is in <code>this</code>.
	 */
	public boolean containsAll(Collection<?> c) {
		for (Object el : c) {
			if (!contains(el)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * MODIFIES: <code>this</code><br>
	 * EFFECTS: <code>this_post = this_pre - c</code>
	 *     Returns true iff any element of <code>c</code> was in <code>this_pre</code>.
	 */
	public boolean removeAll(Collection<?> c) {
		boolean res = false;
		for (Object el: c) {
			if (remove(el)) {
				res = true;
			}
		}
		return res;
	}

	/**
	 * MODIFIES: <code>this</code><br>
	 * EFFECTS: Removes all elements of <code>this</code> that are not elements of <code>c</code>.
	 *    <code>this_post = this_pre INTERSECT c</code>
	 *    Returns true if <code>this_post.size() < this_pre.size()</code> 
	 */
	public boolean retainAll(Collection<?> c) {
		throw new RuntimeException ("Not Implemented");
	}

	public Object[] toArray() {
		throw new RuntimeException ("Not Implemented");
	}

	public <E> E[] toArray(E[] a) {
		throw new RuntimeException ("Not Implemented");
	}
}
