package ps3;

public class StringGraphB implements StringGraph {
	class Edge { 
		   String a, b; 
		   Edge (String p_a, String p_b) {
			   a = p_a;
			   b = p_b;
		   }
		   public boolean equals(Object o) {
			   if (o instanceof Edge) {
				   Edge e = (Edge) o;
				   return e.a.equals(this.a) && e.b.equals(this.b);
			   }
			   return false;
		   }
		   
		   public String toString() {
			   return "<" + a + ", " + b + ">";
		   }
		}

	private Set<String> nodes;
	private Set<Edge> edges;
	// Abstraction Function:
	//    Nodes = nodes
	//    Edges = edges
	// Rep Invariant:
	//     for all e in edges, e.a and e.b are in nodes
	
	public StringGraphB() {
		nodes = new Set<String>();
		edges = new Set<Edge>();
	}
	
	private void checkNode(String s) throws NoNodeException {
		// EFFECTS: If s is a node in this does nothing.  If not,
		//    throws NoNodeException.
		if (!nodes.contains(s)) {
			throw new NoNodeException(s);
		}
	}
	public void addEdge(String s, String t) throws NoNodeException,
			DuplicateException {
		checkNode(s);
		checkNode(t);
		Edge e = new Edge(s, t);
		// This works because we overrode equals to check the edge names
		if (edges.contains(e)) throw new DuplicateException(s + "->" + t);
		edges.add(e);
	}

	public void addNode(String s) throws DuplicateException {
		if (nodes.contains(s)) throw new DuplicateException(s);
		nodes.add(s);
	}

	public Set<String> getAdjacent(String s) throws NoNodeException {
		checkNode(s);
		Set<String> res = new Set<String>();
		for (Edge e : edges) {
			if (e.a.equals(s)) {
				res.add(e.b);
			}
		}
		return res;
	}

	public String toString() {
		return "<" + nodes.toString() + "; " + edges.toString() + ">";
	}
}
